package notifications

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import java.util.concurrent.BlockingQueue
import java.util.concurrent.LinkedBlockingQueue

object NotificationCenter {

    private val queue : BlockingQueue<Notification> = LinkedBlockingQueue()

    val message: Flow<String> = flow {
        while (true) {
            val notification = queue.take()
            emit(notification.message)
            delay(notification.duration.ms)
            emit("")
        }
    }.flowOn(Dispatchers.IO)


    fun addNotification(message: String, duration: NotificationDuration){
        if (message.isNotEmpty()) {
            queue.put(
                Notification(
                    message,
                    duration
                )
            )
        }
    }
}

private data class Notification(
    val message: String,
    val duration: NotificationDuration
)

enum class NotificationDuration(val ms : Long) {
    SHORT(3000L),
    LONG(5000L)
}