package viewModel

import kotlinx.coroutines.channels.BufferOverflow
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableSharedFlow
import notifications.NotificationCenter
import notifications.NotificationDuration
import kotlin.system.exitProcess

object QuickLaunchInputCoordinator {

    private val _uiEvents = MutableSharedFlow<QLUiEvent>(extraBufferCapacity = 1, onBufferOverflow = BufferOverflow.DROP_LATEST)
    val uiEvents : Flow<QLUiEvent> = _uiEvents

    fun enter(userInput: String) {
        if (userInput.isNotEmpty()) {
            if (userInput.startsWith("/")) {
                command(userInput.substring(1))
            } else {
                shortCut(userInput)
            }
        }
    }


    private fun command(cmd: String) {
        when(cmd){
            "x" -> {
                exitProcess(0)
            }
            "v" -> {
                NotificationCenter.addNotification("version c1.0", NotificationDuration.SHORT)
            }
            "s" -> {
                _uiEvents.tryEmit(QLUiEvent.OpenShortcutWindowEvent)
            }
            else -> {
                _uiEvents.tryEmit(QLUiEvent.HelpEvent)
            }
        }

    }

    private fun shortCut(shortCut: String) {
        ShortCutHandler.executeShortcut(shortCut)
    }


    sealed class QLUiEvent {
        object OpenShortcutWindowEvent : QLUiEvent()
        object HelpEvent : QLUiEvent()
    }
}